

//*************************************************************************//
//	Deklaracja parametrow startowych niezbednych do sterowania			   //
//  wyswietlaczem									   					   //
//*************************************************************************//
   #define   lcd_en        portb_3			//pin EN
   #define   lcd_tris_en   trisb_3			//tris EN
   #define   lcd_rs        portb_2			//pin RS
   #define   lcd_tris_rs   trisb_2			//tris RS
   #define   lcd_d7        portb_7			//pin D7
   #define   lcd_tris_d7   trisb_7			//tris D7
   #define   lcd_d6        portb_6			//pin D6
   #define   lcd_tris_d6   trisb_6		//tris D6
   #define   lcd_d5        portb_5			//pin D5
   #define   lcd_tris_d5   trisb_5			//tris D5
   #define   lcd_d4        portb_4			//pin D4
   #define   lcd_tris_d4   trisb_4			//tris D4

#define lcd_type 2           				// 0=5x7, 2 linie
#define lcd_line_two 0x40    				// LCD RAM adres drugiej lini
//*************************************************************************//


//***********************************************************************************//
//  Wysyla polowe bajtu do wyswietlacza												 //
//***********************************************************************************//
void lcd_send_nibble( byte n )
{
   lcd_d4=bit_test(n,0);					//kopiuj 4bity bajtu na odpowiednie piny
   lcd_d5=bit_test(n,1);					//wyswietlacza
   lcd_d6=bit_test(n,2);
   lcd_d7=bit_test(n,3);
   delay_cycles(1);
   lcd_en = 1;								//zapisz do LCD
   delay_us(2);
   lcd_en = 0;
}
//************************************************************************************//

//************************************************************************************//
// Wysyla jeden bajt do LCD															  //
//************************************************************************************//
void lcd_send_byte( byte address, byte n )
{
  lcd_rs = 0;
#ifdef lcd_rw								//jesli komunikacja dwustronna, to
      while ( bit_test(lcd_read_byte(),7) );//czekaj az wyswietlacz bedzie gotowy
#else										//jesli komunikacja jednostronna
      delay_us(40);     					//to odczekaj 40us
      lcd_rs = address;						//jesli address=1. to zapis bajtu jako znak
      delay_cycles(1);						//jesli=0, to zapis jako instrukcja
#ifdef lcd_rw                 				//jesli komunikacja dwustronna, to przelacz
      lcd_rw = 0;							// na zapis
#endif
      delay_cycles(1);
      lcd_en = 0;
      lcd_send_nibble(n >> 4);				//wyslij starsza tetrade bajtu do wyswietlacza
      lcd_send_nibble(n & 0xf);				// i mlodsza
}
//************************************************************************************//

//**************************************************************************************//
//Inicjalizacja wyswietlacza, do pracy 4bitowej  										//
//**************************************************************************************//
void lcd_init()
{byte i;
  lcd_d7=0;									//wpisz 0 na wszystkie piny danych
  lcd_d6=0;
  lcd_d5=0;
  lcd_d4=0;                     			//ustaw piny danych jako wyjscia
  lcd_tris_d7=0;
  lcd_tris_d6=0;
  lcd_tris_d5=0;
  lcd_tris_d4=0;
  lcd_rs = 0;								//zeruj pin RS i ustaw jako wyjscie
  lcd_tris_rs=0;                            //zeruj pin EN i ustaw jako wyjscie
  lcd_en = 0;
  lcd_tris_en=0;
#ifdef lcd_rw								//jesli zdefiniowana komunikacja dwustronna,
    lcd_rw = 0;								//to zeruj pin RW i ustaw jako wyjscie
    lcd_tris_rw=0;
#endif
    delay_ms(15);                           //czekaj na start lcd
    for(i=1;i<=3;++i) {
       lcd_send_nibble(3);					//ustaw tryb 4bitowy
       delay_ms(5);
    }
   lcd_send_nibble(2);						//wyswietlacz dwu liniowy
   lcd_send_byte(0,0x20 | (lcd_type << 2)); // bez kursora
   lcd_send_byte(0,0xc);
   lcd_send_byte(0,1);
   delay_ms(2);
   lcd_send_byte(0,6);
}
//*************************************************************************************//

//******************************************************************************//
// ustawia kursor na pozycji xy													//
//******************************************************************************//
void lcd_gotoxy( byte y, byte x)
{byte address;
 if(y!=1)
     address=lcd_line_two;
 else
   address=0;
   address+=x-1;
   lcd_send_byte(0,0x80|address);
}
//********************************************************************************//

//********************************************************************************//
//    wyswietla znaki na wyswietlaczu											  //
//********************************************************************************//
void lcd_putc(char f)
{
switch (f)
   {
   case '\r'     : lcd_send_byte(0,1);					//kasuj LCD
                   delay_ms(2);
                   break;
   case '\n'     : lcd_gotoxy(2,1);            break;	//nowa linia LCD
   default       : lcd_send_byte(1,f);         break;	//wyswietl znak
   }
}
//*******************************************************************************//

//*******************************************************************************//
// Wyswietla bajt jako dwie cyfry BCD											 //
//*******************************************************************************//
void lcd_putbcd( int cd)
  {
  lcd_send_byte(1, (cd/16)+'0' );
  lcd_send_byte(1, (cd%16)+'0' );
  }
//*******************************************************************************//

//*******************************************************************************//
// Wyswietla bajt dziesietnie , max. wartosc= 99								 //
//*******************************************************************************//
Void lcd_putdec(int de)
{
  lcd_send_byte(1, (de/10)+'0' );
  lcd_send_byte(1, (de%10)+'0' );
}
//*******************************************************************************//
